﻿using System;

namespace Billing
{
    public static class LocationExtension
    {
        private const double PI = 3.1415926535897931;

        public static (double longitude, double latitude) Wgs84ToGcj02(this (double lon, double lat) position)
        {
            if (IsOutOfChina(position.lon, position.lat))
            {
                return position;
            }

            var a = 6378245d;
            var ee = 0.00669342162296594323;

            var offsetLatitude = TransformLatitude(position.lon - 105d, position.lat - 35d);
            var offsetLongitude = TransformLongitude(position.lon - 105d, position.lat - 35d);
            var radiusLatitude = position.lat / 180d * PI;
            var magic = 1d - ee * Math.Pow(Math.Sin(radiusLatitude), 2);
            var sqrtMagic = Math.Sqrt(magic);
            offsetLatitude = offsetLatitude * 180d / (a * (1d - ee) / (magic * sqrtMagic) * PI);
            offsetLongitude = offsetLongitude * 180d / (a / sqrtMagic * Math.Cos(radiusLatitude) * PI);
            return (position.lon + offsetLongitude, position.lat + offsetLatitude);
        }

        private static bool IsOutOfChina(double lon, double lat)
        {
            return lon < 72.004 || lon > 137.8347
                || lat < 0.8293 || lat > 55.8171;
        }

        private static double TransformLatitude(double x, double y)
        {
            var ret = -100d + 2d * x + 3d * y + 0.2 * y * y + 0.1 * x * y + 0.2 * Math.Sqrt(Math.Abs(x));
            ret += (20d * Math.Sin(6d * x * PI) + 20d * Math.Sin(2d * x * PI)) * 2d / 3d;
            ret += (20d * Math.Sin(y * PI) + 40d * Math.Sin(y / 3d * PI)) * 2d / 3d;
            ret += (160d * Math.Sin(y / 12d * PI) + 320d * Math.Sin(y * PI / 30d)) * 2d / 3d;
            return ret;
        }

        private static double TransformLongitude(double x, double y)
        {
            var ret = 300d + x + 2d * y + 0.1 * x * x + 0.1 * x * y + 0.1 * Math.Sqrt(Math.Abs(x));
            ret += (20d * Math.Sin(6d * x * PI) + 20d * Math.Sin(2d * x * PI)) * 2d / 3d;
            ret += (20d * Math.Sin(x * PI) + 40d * Math.Sin(x / 3d * PI)) * 2d / 3d;
            ret += (150d * Math.Sin(x / 12d * PI) + 300d * Math.Sin(x / 30d * PI)) * 2d / 3d;
            return ret;
        }

    }
}
